@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern::Document::OpenAPI; # git description: v0.015-3-g4693d99
+package JSON::Schema::Modern::Document::OpenAPI; # git description: v0.016-16-g5fee8e7
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One OpenAPI v3.1 document
 # KEYWORDS: JSON Schema data validation request response OpenAPI
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -15,7 +15,7 @@ use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use JSON::Schema::Modern::Utilities 0.525 qw(assert_keyword_exists assert_keyword_type E canonical_uri);
+use JSON::Schema::Modern::Utilities 0.525 qw(assert_keyword_exists assert_keyword_type E canonical_uri get_type);
 use Safe::Isa;
 use File::ShareDir 'dist_dir';
 use Path::Tiny;
@@ -84,6 +84,11 @@ sub traverse ($self, $evaluator) {
     vocabularies => [],
   };
 
+  if ((my $type = get_type($schema)) ne 'object') {
+    ()= E($state, 'invalid document type: %s', $type);
+    return $state;
+  }
+
   # /openapi: https://spec.openapis.org/oas/v3.1.0#openapi-object
   return $state if not assert_keyword_exists({ %$state, keyword => 'openapi' }, $schema)
     or not assert_keyword_type({ %$state, keyword => 'openapi' }, $schema, 'string');
@@ -230,7 +235,7 @@ JSON::Schema::Modern::Document::OpenAPI - One OpenAPI v3.1 document
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 SYNOPSIS
 
